"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var abstract_repository_1 = require("./abstract-repository");
var boot_pool_dao_1 = require("../dao/boot-pool-dao");
var boot_environment_dao_1 = require("../dao/boot-environment-dao");
var model_1 = require("../model");
var model_event_name_1 = require("../model-event-name");
var BootPoolRepository = (function (_super) {
    __extends(BootPoolRepository, _super);
    function BootPoolRepository(bootPoolDao, bootEnvironmentDao) {
        var _this = _super.call(this, [model_1.Model.BootEnvironment, model_1.Model.BootPool]) || this;
        _this.bootPoolDao = bootPoolDao;
        _this.bootEnvironmentDao = bootEnvironmentDao;
        return _this;
    }
    BootPoolRepository.getInstance = function () {
        if (!BootPoolRepository.instance) {
            BootPoolRepository.instance = new BootPoolRepository(new boot_pool_dao_1.BootPoolDao(), new boot_environment_dao_1.BootEnvironmentDao());
        }
        return BootPoolRepository.instance;
    };
    BootPoolRepository.prototype.getBootPoolConfig = function () {
        return this.bootPool ? Promise.resolve(this.bootPool) : this.bootPoolDao.get();
    };
    BootPoolRepository.prototype.listBootEnvironments = function () {
        return this.bootEnvironments;
    };
    BootPoolRepository.prototype.loadBootEnvironments = function () {
        return this.bootEnvironments ? Promise.resolve() : this.bootEnvironmentDao.list();
    };
    BootPoolRepository.prototype.deleteBootEnvironment = function (bootEnvironment) {
        return this.bootEnvironmentDao.delete(bootEnvironment);
    };
    BootPoolRepository.prototype.scrubBootPool = function () {
        return this.bootPoolDao.scrub();
    };
    BootPoolRepository.prototype.activateBootEnvironment = function (bootEnvironment) {
        return this.bootEnvironmentDao.activate(bootEnvironment);
    };
    BootPoolRepository.prototype.saveBootEnvironment = function (bootEnvironment) {
        return this.bootEnvironmentDao.save(bootEnvironment);
    };
    BootPoolRepository.prototype.cloneBootEnvironment = function (bootEnvironment) {
        var cloneName = this.getCloneName(bootEnvironment);
        return this.bootEnvironmentDao.clone(bootEnvironment, cloneName);
    };
    BootPoolRepository.prototype.handleStateChange = function (name, state, overlay) {
        switch (name) {
            case model_1.Model.BootPool:
                this.bootPool = state.get().toJS();
                this.eventDispatcherService.dispatch(model_event_name_1.ModelEventName.BootPool.contentChange, this.bootPool);
                break;
            default:
                this.bootEnvironments = this.dispatchModelEvents(this.bootEnvironments, model_event_name_1.ModelEventName.BootEnvironment, state, overlay);
                break;
        }
    };
    BootPoolRepository.prototype.handleEvent = function (name, data) { };
    BootPoolRepository.prototype.getCloneName = function (bootEnvironment) {
        var commonPrefix = bootEnvironment.id + '-copy-', usedNames = _.filter(this.bootEnvironments.keySeq().toJS(), function (name) { return _.startsWith(commonPrefix); }), lastUsedIndex = !usedNames.length ?
            0 :
            _.last(_.sortBy(_.map(usedNames, function (name) { return _.toNumber(name.substring(commonPrefix.length)); })));
        return commonPrefix + _.toString(lastUsedIndex + 1);
    };
    return BootPoolRepository;
}(abstract_repository_1.AbstractRepository));
exports.BootPoolRepository = BootPoolRepository;
