"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var abstract_repository_1 = require("./abstract-repository");
var model_event_name_1 = require("../model-event-name");
var replication_dao_1 = require("../dao/replication-dao");
var compress_replication_transport_option_dao_1 = require("../dao/compress-replication-transport-option-dao");
var encrypt_replication_transport_option_dao_1 = require("../dao/encrypt-replication-transport-option-dao");
var throttle_replication_transport_option_dao_1 = require("../dao/throttle-replication-transport-option-dao");
var model_1 = require("../model");
var _ = require("lodash");
var ReplicationRepository = (function (_super) {
    __extends(ReplicationRepository, _super);
    function ReplicationRepository(replicationDao, compressReplicationTransportOptionDao, encryptReplicationTransportOptionDao, throttleReplicationTransportOptionDao) {
        var _this = _super.call(this, [model_1.Model.Replication]) || this;
        _this.replicationDao = replicationDao;
        _this.compressReplicationTransportOptionDao = compressReplicationTransportOptionDao;
        _this.encryptReplicationTransportOptionDao = encryptReplicationTransportOptionDao;
        _this.throttleReplicationTransportOptionDao = throttleReplicationTransportOptionDao;
        return _this;
    }
    ReplicationRepository.getInstance = function () {
        if (!ReplicationRepository.instance) {
            ReplicationRepository.instance = new ReplicationRepository(new replication_dao_1.ReplicationDao(), new compress_replication_transport_option_dao_1.CompressReplicationTransportOptionDao(), new encrypt_replication_transport_option_dao_1.EncryptReplicationTransportOptionDao(), new throttle_replication_transport_option_dao_1.ThrottleReplicationTransportOptionDao());
        }
        return ReplicationRepository.instance;
    };
    ReplicationRepository.prototype.listReplications = function () {
        return this.replicationDao.list();
    };
    ReplicationRepository.prototype.listReplicationsForVolume = function (volume) {
        return Promise.all([
            this.replicationDao.list(),
            this.getHostUuid()
        ]).spread(function (replications, host) { return _.filter(replications, function (replication) {
            return (replication.master === host && replication.datasets[0].master.startsWith(volume)) ||
                (replication.slave === host && replication.datasets[0].slave.startsWith(volume));
        }); });
    };
    ReplicationRepository.prototype.getNewReplicationInstance = function () {
        return this.replicationDao.getNewInstance();
    };
    ReplicationRepository.prototype.getNewReplicationTransportOptionInstance = function (type) {
        if (type === model_1.Model.CompressReplicationTransportOption) {
            return this.compressReplicationTransportOptionDao.getNewInstance();
        }
        else if (type === model_1.Model.EncryptReplicationTransportOption) {
            return this.encryptReplicationTransportOptionDao.getNewInstance();
        }
        else if (type === model_1.Model.ThrottleReplicationTransportOption) {
            return this.throttleReplicationTransportOptionDao.getNewInstance();
        }
    };
    ReplicationRepository.prototype.saveReplication = function (replication) {
        return this.replicationDao.save(replication);
    };
    ReplicationRepository.prototype.deleteReplication = function (replication) {
        return this.replicationDao.delete(replication);
    };
    ReplicationRepository.prototype.syncReplication = function (replicationId) {
        return this.replicationDao.sync(replicationId);
    };
    ReplicationRepository.prototype.getHostUuid = function () {
        return this.replicationDao.getHostUuid();
    };
    ReplicationRepository.prototype.handleStateChange = function (name, state) {
        switch (name) {
            case 'Replication':
                this.replications = this.dispatchModelEvents(this.replications, model_event_name_1.ModelEventName.Replication, state);
                break;
            default:
                break;
        }
    };
    ReplicationRepository.prototype.getReplicationId = function (replication) {
        return replication._replication ? replication._replication.id :
            replication.id ? replication.id : replication;
    };
    ReplicationRepository.prototype.handleEvent = function (name, data) {
    };
    return ReplicationRepository;
}(abstract_repository_1.AbstractRepository));
exports.ReplicationRepository = ReplicationRepository;
