"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var replication_repository_1 = require("../repository/replication-repository");
var abstract_route_1 = require("./abstract-route");
var model_1 = require("../model");
var calendar_repository_1 = require("../repository/calendar-repository");
var ReplicationRoute = (function (_super) {
    __extends(ReplicationRoute, _super);
    function ReplicationRoute(replicationRepository, calendarRepository) {
        var _this = _super.call(this) || this;
        _this.replicationRepository = replicationRepository;
        _this.calendarRepository = calendarRepository;
        _this.objectType = model_1.Model.Replication;
        return _this;
    }
    ReplicationRoute.getInstance = function () {
        if (!ReplicationRoute.instance) {
            ReplicationRoute.instance = new ReplicationRoute(replication_repository_1.ReplicationRepository.getInstance(), calendar_repository_1.CalendarRepository.getInstance());
        }
        return ReplicationRoute.instance;
    };
    ReplicationRoute.prototype.list = function (volumeId, stack) {
        var self = this, columnIndex = 2;
        return this.loadListInColumn(stack, columnIndex, columnIndex - 1, '/replication', model_1.Model.Replication, this.replicationRepository.listReplicationsForVolume(volumeId));
    };
    ReplicationRoute.prototype.create = function (volumeId, stack) {
        var self = this, objectType = model_1.Model.Replication, columnIndex = 3, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/create',
            volume: volumeId
        };
        return Promise.all([
            this.replicationRepository.getNewReplicationInstance(),
            this.modelDescriptorService.getUiDescriptorForType(objectType),
            this.replicationRepository.getHostUuid()
        ]).spread(function (replication, uiDescriptor, host) {
            context.object = replication;
            context.object.master = host;
            context.object.datasets = [{}];
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    ReplicationRoute.prototype.createForDataset = function (datasetId, stack) {
        var self = this, objectType = model_1.Model.Replication, columnIndex = 4, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/replication',
            dataset: datasetId
        };
        return Promise.all([
            this.replicationRepository.getNewReplicationInstance(),
            this.modelDescriptorService.getUiDescriptorForType(objectType),
            this.replicationRepository.getHostUuid()
        ]).spread(function (replication, uiDescriptor, host) {
            context.object = replication;
            context.object.master = host;
            context.object.datasets = [{}];
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    ReplicationRoute.prototype.get = function (replicationId, stack) {
        var self = this, objectType = model_1.Model.Replication, columnIndex = 3, parentContext = stack[columnIndex - 1], context = {
            columnIndex: columnIndex,
            objectType: this.objectType,
            parentContext: parentContext,
            path: parentContext.path + '/_/' + encodeURIComponent(replicationId)
        };
        return Promise.all([
            this.replicationRepository.listReplications(),
            this.modelDescriptorService.getUiDescriptorForType(objectType),
            this.calendarRepository.listTasks()
        ]).spread(function (replication, uiDescriptor, calendarTasks) {
            context.object = _.find(replication, { id: replicationId });
            context.object._calendarTask = _.find(calendarTasks, function (calendarTask) {
                return calendarTask['args'][0] === context.object.id;
            });
            context.userInterfaceDescriptor = uiDescriptor;
            return self.updateStackWithContext(stack, context);
        });
    };
    return ReplicationRoute;
}(abstract_route_1.AbstractRoute));
exports.ReplicationRoute = ReplicationRoute;
