"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var volume_repository_1 = require("../repository/volume-repository");
var disk_repository_1 = require("../repository/disk-repository");
var abstract_route_1 = require("./abstract-route");
var model_1 = require("../model");
var _ = require("lodash");
var VolumeRoute = (function (_super) {
    __extends(VolumeRoute, _super);
    function VolumeRoute(volumeRepository, diskRepository) {
        var _this = _super.call(this) || this;
        _this.volumeRepository = volumeRepository;
        _this.diskRepository = diskRepository;
        return _this;
    }
    VolumeRoute.getInstance = function () {
        if (!VolumeRoute.instance) {
            VolumeRoute.instance = new VolumeRoute(volume_repository_1.VolumeRepository.getInstance(), disk_repository_1.DiskRepository.getInstance());
        }
        return VolumeRoute.instance;
    };
    VolumeRoute.prototype.get = function (volumeId, stack) {
        var columnIndex = 1;
        return this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, abstract_route_1.AbstractRoute.getObjectPathSuffix(model_1.Model.Volume, volumeId), model_1.Model.Volume, this.volumeRepository.listVolumes(), { id: volumeId });
    };
    VolumeRoute.prototype.topology = function (volumeId, stack) {
        var _this = this;
        var columnIndex = 2;
        return Promise.all([
            this.loadPropertyInColumn(stack, columnIndex, columnIndex - 1, '/topology', model_1.Model.ZfsTopology, 'topology'),
            this.volumeRepository.listVolumes()
        ]).spread(function (stack, volumes) {
            _.last(stack).object._volume = _.find(volumes, { id: volumeId });
            _.last(stack).object._disks = _this.diskRepository.listAvailableDisks();
            return stack;
        });
    };
    VolumeRoute.prototype.topologyDisk = function (volumeId, diskId, stack) {
        var columnIndex = 3;
        return Promise.all([
            this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, '/disk', model_1.Model.Disk, this.diskRepository.listDisks(), { id: diskId }),
            this.volumeRepository.listVolumes()
        ]).spread(function (stack, volumes) {
            _.last(stack).object._volume = _.find(volumes, { id: volumeId });
            return stack;
        });
    };
    VolumeRoute.prototype.creatorDisk = function (diskId, stack) {
        var columnIndex = 2;
        return this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, '/disk', model_1.Model.Disk, this.diskRepository.listDisks(), { id: diskId });
    };
    VolumeRoute.prototype.create = function (stack) {
        var columnIndex = 1;
        return this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, '/create', model_1.Model.Volume, this.volumeRepository.getNewVolume());
    };
    VolumeRoute.prototype.import = function (stack) {
        var columnIndex = 1;
        return this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, abstract_route_1.AbstractRoute.getObjectPathSuffix(model_1.Model.VolumeImporter, '-'), model_1.Model.VolumeImporter, this.volumeRepository.getVolumeImporter());
    };
    VolumeRoute.prototype.mediaImport = function (stack) {
        var columnIndex = 1;
        return this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, abstract_route_1.AbstractRoute.getObjectPathSuffix(model_1.Model.VolumeMediaImporter, '-'), model_1.Model.VolumeMediaImporter, this.volumeRepository.getVolumeMediaImporter());
    };
    VolumeRoute.prototype.getDetachedVolume = function (volumeId, stack) {
        var columnIndex = 2;
        return this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, abstract_route_1.AbstractRoute.getObjectPathSuffix(model_1.Model.DetachedVolume, volumeId), model_1.Model.DetachedVolume, this.volumeRepository.listDetachedVolumes(), { id: _.toString(volumeId) });
    };
    VolumeRoute.prototype.getDetachedVolumeTopology = function (stack) {
        var columnIndex = 3;
        return this.loadPropertyInColumn(stack, columnIndex, columnIndex - 1, '/topology', model_1.Model.ZfsTopology, 'topology').then(function (stack) {
            _.last(stack).object._isDetached = true;
            return stack;
        });
    };
    VolumeRoute.prototype.importEncrypted = function (stack) {
        var columnIndex = 2;
        return this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, '/encrypted', model_1.Model.EncryptedVolumeImporter, this.volumeRepository.getEncryptedVolumeImporterInstance());
    };
    VolumeRoute.prototype.getEncryptedVolumeActions = function (volumeId, stack) {
        var columnIndex = 2;
        return this.loadObjectInColumn(stack, columnIndex, columnIndex - 1, '/encrypted-volume-actions', model_1.Model.EncryptedVolumeActions, this.volumeRepository.getEncryptedVolumeActionsForVolume(volumeId));
    };
    return VolumeRoute;
}(abstract_route_1.AbstractRoute));
exports.VolumeRoute = VolumeRoute;
