"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var account_repository_1 = require("../repository/account-repository");
var AccountService = (function () {
    function AccountService(accountRepository) {
        this.accountRepository = accountRepository;
    }
    AccountService.getInstance = function () {
        if (!AccountService.instance) {
            AccountService.instance = new AccountService(account_repository_1.AccountRepository.getInstance());
        }
        return AccountService.instance;
    };
    AccountService.prototype.listLocalUsers = function (options) {
        return this.handleSearchQuery(this.accountRepository.listLocalUsers(), options);
    };
    AccountService.prototype.listLocalGroups = function (options) {
        return this.handleSearchQuery(this.accountRepository.listLocalGroups(), options);
    };
    AccountService.prototype.listSystemUsers = function () {
        return this.accountRepository.listSystemUsers();
    };
    AccountService.prototype.listSystemGroups = function () {
        return this.accountRepository.listSystemGroups();
    };
    AccountService.prototype.searchUser = function (value, options) {
        return this.handleSearchQuery(this.accountRepository.searchUser(value), options);
    };
    AccountService.prototype.searchUserWithCriteria = function (criteria, options) {
        return this.handleSearchQuery(this.accountRepository.searchUserWithCriteria(criteria), options);
    };
    AccountService.prototype.searchGroup = function (value, options) {
        return this.handleSearchQuery(this.accountRepository.searchGroup(value), options);
    };
    AccountService.prototype.searchGroupWithCriteria = function (criteria, options) {
        return this.handleSearchQuery(this.accountRepository.searchGroupWithCriteria(criteria), options);
    };
    AccountService.prototype.handleSearchQuery = function (searchQuery, options) {
        return searchQuery.then(function (entries) {
            if (options && options.labelPath && options.valuePath) {
                entries = entries.map(function (entry) {
                    return {
                        label: entry[options.labelPath],
                        value: entry[options.valuePath]
                    };
                });
            }
            return entries;
        });
    };
    return AccountService;
}());
exports.AccountService = AccountService;
