"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var event_dispatcher_service_1 = require("./event-dispatcher-service");
var model_event_name_1 = require("../model-event-name");
var uuid = require("uuid");
var _ = require("lodash");
var SubmittedTask_1 = require("../model/SubmittedTask");
var Events_1 = require("../Events");
var MiddlewareClient = (function () {
    function MiddlewareClient(eventDispatcherService) {
        this.eventDispatcherService = eventDispatcherService;
        this.REQUEST_TIMEOUT = 90000;
        this.handlers = new Map();
    }
    MiddlewareClient.getInstance = function () {
        if (!MiddlewareClient.instance) {
            MiddlewareClient.instance = new MiddlewareClient(event_dispatcher_service_1.EventDispatcherService.getInstance());
        }
        return MiddlewareClient.instance;
    };
    MiddlewareClient.prototype.connect = function (url) {
        url = url || MiddlewareClient.getRootURL('ws') + '/dispatcher/socket';
        if (this.socket) {
            if (this.socket.url !== url && this.socket.readyState === this.socket.OPEN) {
                this.closeConnection();
            }
            this.socket = null;
        }
        return this.openConnection(url);
    };
    MiddlewareClient.prototype.login = function (login, password) {
        var _this = this;
        this.state = MiddlewareClient.CONNECTING;
        return this.connectionPromise.then(function () {
            var payload = {
                namespace: 'rpc',
                name: 'auth',
                args: {
                    username: login,
                    password: password
                }
            };
            return _this.send(payload);
        }).then(function () {
            _this.url = MiddlewareClient.getHost();
            _this.user = login;
            _this.eventDispatcherService.dispatch(Events_1.Events.userLoggedIn, {
                url: _this.url,
                username: _this.user
            });
            _this.state = MiddlewareClient.OPEN;
            _this.dispatchConnectionStatus();
            return _this.state;
        }, function (error) {
            if (error) {
                if (error.name === 'MiddlewareError') {
                    var message = error.middlewareMessage;
                    if (message.args) {
                        if (message.args.message) {
                            throw new Error(message.args.message);
                        }
                    }
                }
            }
        });
    };
    MiddlewareClient.prototype.continueRpcMethod = function (id, sequence) {
        var _this = this;
        return this.connectionPromise.then(function () {
            var payload = {
                namespace: 'rpc',
                name: 'continue',
                id: id,
                args: sequence
            };
            return _this.send(payload);
        });
    };
    MiddlewareClient.prototype.enableStreamingResponses = function () {
        return this.callRpcMethod('management.enable_features', [['streaming_responses']]);
    };
    MiddlewareClient.prototype.callRpcMethod = function (name, args) {
        var self = this;
        return this.connectionPromise.then(function () {
            var payload = {
                namespace: 'rpc',
                name: 'call',
                args: {
                    method: name,
                    args: args || []
                }
            };
            return self.send(payload);
        });
    };
    MiddlewareClient.prototype.submitTask = function (name, args) {
        var self = this, temporaryTaskId = uuid.v4();
        this.eventDispatcherService.dispatch('taskSubmitted', temporaryTaskId);
        return this.callRpcMethod('task.submit', [
            name,
            args || []
        ]).then(function (taskId) {
            self.eventDispatcherService.dispatch('taskCreated', {
                old: temporaryTaskId,
                new: taskId
            });
            return new SubmittedTask_1.SubmittedTask(taskId, self.getTaskPromise(taskId));
        });
    };
    MiddlewareClient.prototype.submitTaskWithDownload = function (name, args) {
        var self = this, temporaryTaskId = uuid.v4();
        this.eventDispatcherService.dispatch('taskSubmitted', temporaryTaskId);
        return this.callRpcMethod('task.submit_with_download', [
            name,
            args || []
        ]).spread(function (taskId, links) {
            self.eventDispatcherService.dispatch('taskCreated', {
                old: temporaryTaskId,
                new: taskId
            });
            return {
                taskId: taskId,
                taskPromise: self.getTaskPromise(taskId),
                link: MiddlewareClient.getRootURL('http') + links[0]
            };
        });
    };
    MiddlewareClient.prototype.getTaskPromise = function (taskId) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var eventListener = _this.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName.Task.change(taskId), function (task) {
                switch (task.get('state')) {
                    case 'FINISHED':
                        resolve(task.get('result'));
                        _this.eventDispatcherService.removeEventListener(model_event_name_1.ModelEventName.Task.change(taskId), eventListener);
                        break;
                    case 'FAILED':
                        reject(task.get('error').toJS());
                        _this.eventDispatcherService.removeEventListener(model_event_name_1.ModelEventName.Task.change(taskId), eventListener);
                        break;
                    case 'ABORTED':
                        reject('Aborted');
                        _this.eventDispatcherService.removeEventListener(model_event_name_1.ModelEventName.Task.change(taskId), eventListener);
                        break;
                }
            });
        });
    };
    MiddlewareClient.prototype.submitTaskWithUpload = function (name, args, file) {
        var self = this, temporaryTaskId = uuid.v4();
        this.eventDispatcherService.dispatch('taskSubmitted', temporaryTaskId);
        return this.callRpcMethod('task.submit_with_upload', [name, args]).spread(function (taskId, tokens) {
            self.eventDispatcherService.dispatch('taskCreated', {
                old: temporaryTaskId,
                new: taskId
            });
            self.sendFileWithToken(file, tokens[0]);
        });
    };
    MiddlewareClient.prototype.uploadFile = function (file, destination, mode) {
        if (mode === void 0) { mode = '755'; }
        var self = this;
        return this.callRpcMethod('filesystem.upload', ['/root/' + file.name, file.size, mode]).then(function (response) {
            var token = Array.isArray(response) ? response[1][0] : response;
            self.sendFileWithToken(file, token);
        });
    };
    MiddlewareClient.prototype.sendFileWithToken = function (file, token) {
        var _this = this;
        var connection = new WebSocket(MiddlewareClient.getRootURL('ws') + '/dispatcher/file');
        connection.onopen = function () {
            connection.send(JSON.stringify({ token: token }));
        };
        connection.onmessage = function (event) {
            var message = JSON.parse(event.data), filePos = 0;
            if (message.status === "ok") {
                console.log("Fileconnection authenticated for upload");
                _this.sendChunkOnConnection(connection, file, filePos, filePos + MiddlewareClient.BUFSIZE);
                connection.onmessage = function (event) { return _this.handleMessage(event, _this.url); };
            }
        };
    };
    MiddlewareClient.prototype.sendChunkOnConnection = function (connection, file, start, stop) {
        var _this = this;
        start = start || 0;
        stop = stop || file.size;
        var reader = new FileReader();
        if (stop > file.size) {
            stop = file.size;
        }
        reader.onloadend = function (event) {
            var target = event.target;
            if (target.readyState === MiddlewareClient.FILE_READER_DONE) {
                connection.send(target.result);
                if (stop == file.size) {
                    connection.send(JSON.stringify({ 'done': true }));
                }
                else if (stop + MiddlewareClient.BUFSIZE < file.size) {
                    _this.sendChunkOnConnection(connection, file, stop, stop + MiddlewareClient.BUFSIZE);
                }
                else {
                    _this.sendChunkOnConnection(connection, file, stop, file.size);
                }
                ;
            }
        };
        var blob = file.slice(start, stop);
        reader.readAsArrayBuffer(blob);
    };
    MiddlewareClient.prototype.subscribeToEvents = function (name) {
        return this.setEventSubscription(typeof name === 'string' ? [name] : name, 'subscribe');
    };
    MiddlewareClient.prototype.unsubscribeFromEvents = function (name) {
        return this.setEventSubscription(typeof name === 'string' ? [name] : name, 'unsubscribe');
    };
    MiddlewareClient.prototype.getExplicitHostParam = function () {
        return MiddlewareClient.getHostParam() || '';
    };
    MiddlewareClient.prototype.setEventSubscription = function (name, status) {
        var self = this, payload = {
            namespace: 'events',
            name: status,
            id: uuid.v4(),
            args: name
        };
        return this.connectionPromise.then(function () {
            return self.socket.send(JSON.stringify(payload));
        });
    };
    MiddlewareClient.prototype.send = function (payload) {
        var self = this;
        return this.connectionPromise.then(function () {
            var resolve, reject, promise = new Promise(function (_resolve, _reject) {
                resolve = _resolve;
                reject = _reject;
            });
            if (!payload.id) {
                payload.id = uuid.v4();
            }
            self.socket.send(JSON.stringify(payload));
            self.handlers.set(payload.id, {
                payload: payload,
                resolve: resolve,
                reject: reject,
                promise: promise
            });
            return promise.timeout(self.REQUEST_TIMEOUT).catch(function (error) {
                if (error) {
                    if (error.name === 'TimeoutError') {
                        console.log('[' + self.url + '] Request timeout: ' + JSON.stringify(payload));
                        self.handlers.get(payload.id).reject();
                        self.handlers.delete(payload.id);
                    }
                }
                console.log(error, payload);
                throw error;
            });
        });
    };
    MiddlewareClient.prototype.closeConnection = function () {
        console.log('Closing connection to ' + this.socket.url);
        this.socket.close(MiddlewareClient.WEBSOCKET_CLOSE_NORMAL);
        this.state = MiddlewareClient.CLOSED;
    };
    MiddlewareClient.prototype.openConnection = function (url) {
        var _this = this;
        if (!this.socket) {
            this.connectionPromise = new Promise(function (resolve, reject) {
                console.log('Opening connection to ' + url);
                var isResolved = false;
                _this.socket = new WebSocket(url);
                _this.socket.onopen = function () {
                    isResolved = true;
                    resolve();
                };
                _this.socket.onmessage = function (event) { return _this.handleMessage(event, _this.url); };
                _this.socket.onerror = function (event) {
                    if (!isResolved) {
                        reject(new MiddlewareError({
                            args: {
                                message: 'Server connection error'
                            }
                        }));
                    }
                    _this.handleError(event, _this.url);
                };
                _this.socket.onclose = function (event) { return _this.handleClose(event, _this.url); };
            }).then(function () {
                _this.state = MiddlewareClient.CONNECTED;
                return _this.dispatchConnectionStatus();
            });
        }
        return this.connectionPromise;
    };
    MiddlewareClient.prototype.dispatchConnectionStatus = function () {
        this.eventDispatcherService.dispatch('connectionStatusChange', this.state);
    };
    MiddlewareClient.prototype.handleError = function (event, url) {
        console.warn('[' + url + '] WS connection error:', event);
        this.dispatchConnectionStatus();
        this.state = MiddlewareClient.CLOSED;
        this.connectionPromise = null;
        this.socket = null;
    };
    MiddlewareClient.prototype.handleClose = function (event, url) {
        console.log('[' + url + '] WS connection closed', event);
        this.dispatchConnectionStatus();
        this.state = MiddlewareClient.CLOSED;
        this.connectionPromise = null;
        this.socket = null;
        setTimeout(function () {
            if (window.location.hash.length === 0) {
                window.location.hash = '#';
            }
            if (window.location.hash.indexOf(';disconnected') === -1) {
                window.location.hash += ';disconnected';
            }
            location.reload();
        }, MiddlewareClient.DISCONNECT_REDIRECT_TIMEOUT);
    };
    MiddlewareClient.prototype.handleMessage = function (event, url) {
        try {
            var message = JSON.parse(event.data);
            if (message.namespace === 'rpc') {
                switch (message.name) {
                    case 'response':
                        this.handleRpcResponse(message);
                        break;
                    case 'error':
                        this.handleRpcError(message);
                        break;
                    case 'fragment':
                    case 'end':
                        this.handleFragmentResponse(message);
                        break;
                    default:
                        break;
                }
            }
            else if (message.namespace === 'events' && message.name === 'event') {
                this.handleEvent(message.args.name, message.args);
            }
            else if (message.namespace === 'events' && message.name === 'event_burst') {
                this.handleEventBurst(message.args.events);
            }
        }
        catch (error) {
            console.warn('[' + url + '] Unable to handle message: -' + event.data + '-');
        }
    };
    MiddlewareClient.prototype.handleRpcResponse = function (message) {
        if (this.handlers.has(message.id)) {
            var deferred = this.handlers.get(message.id);
            this.handlers.delete(message.id);
            deferred.resolve(message.args);
        }
    };
    MiddlewareClient.prototype.handleRpcError = function (message) {
        if (this.handlers.has(message.id)) {
            var deferred = this.handlers.get(message.id);
            this.handlers.delete(message.id);
            deferred.reject(new MiddlewareError(message));
        }
    };
    MiddlewareClient.prototype.handleFragmentResponse = function (message) {
        var messageId = message.id;
        if (this.handlers.has(messageId)) {
            var deferred = this.handlers.get(messageId);
            this.handlers.delete(messageId);
            deferred.resolve(message);
        }
    };
    MiddlewareClient.prototype.handleEvent = function (name, event) {
        if (_.startsWith(name, 'entity-subscriber.')) {
            this.eventDispatcherService.dispatch('middlewareModelChange', event.args);
        }
        else if (_.startsWith(name, 'statd.')) {
            this.eventDispatcherService.dispatch('statsChange', event);
        }
    };
    MiddlewareClient.prototype.handleEventBurst = function (events) {
        var _this = this;
        if (events && events.length > 0) {
            _.forEach(events, function (event) {
                _this.handleEvent(event.name, event);
            });
        }
    };
    MiddlewareClient.getRootURL = function (protocol) {
        var scheme = protocol + (location.protocol === 'https:' ? 's' : ''), host = MiddlewareClient.getHost();
        return scheme + "://" + host;
    };
    MiddlewareClient.getHost = function () {
        var result = location.host, hostParam = MiddlewareClient.getHostParam();
        if (hostParam) {
            var host = hostParam.split('=')[1];
            if (host && host.length > 0) {
                result = _.head(_.split(host, ';'));
            }
        }
        return result;
    };
    MiddlewareClient.getHostParam = function () {
        return _.find(_.split(location.href, /[?&]/), function (param) { return _.startsWith(param, 'host='); });
    };
    return MiddlewareClient;
}());
MiddlewareClient.FILE_READER_DONE = 2;
MiddlewareClient.BUFSIZE = 1048576;
MiddlewareClient.WEBSOCKET_CLOSE_NORMAL = 1000;
MiddlewareClient.UNKNOWN_CONNECTION_STATUS = 99;
MiddlewareClient.CONNECTING = 'CONNECTING';
MiddlewareClient.OPEN = 'OPEN';
MiddlewareClient.CONNECTED = 'CONNECTED';
MiddlewareClient.CLOSED = 'CLOSED';
MiddlewareClient.DISCONNECT_REDIRECT_TIMEOUT = 2000;
exports.MiddlewareClient = MiddlewareClient;
var MiddlewareError = (function (_super) {
    __extends(MiddlewareError, _super);
    function MiddlewareError(middlewareMessage) {
        var _this = _super.call(this) || this;
        _this.name = 'MiddlewareError';
        _this.message = middlewareMessage.args.message;
        _this.middlewareMessage = middlewareMessage;
        return _this;
    }
    return MiddlewareError;
}(Error));
