"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var abstract_section_service_ng_1 = require("./abstract-section-service-ng");
var network_repository_1 = require("../../repository/network-repository");
var system_repository_1 = require("../../repository/system-repository");
var NetworkInterfaceAliasType_1 = require("../../model/enumerations/NetworkInterfaceAliasType");
var NetworkInterfaceType_1 = require("../../model/enumerations/NetworkInterfaceType");
var _ = require("lodash");
var NetworkInterfaceRepository_1 = require("../../repository/NetworkInterfaceRepository");
var model_event_name_1 = require("../../model-event-name");
var NetworkSectionService = (function (_super) {
    __extends(NetworkSectionService, _super);
    function NetworkSectionService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.INTERFACE_TYPES = network_repository_1.NetworkRepository.INTERFACE_TYPES;
        _this.IPV4_DEFAULT_NETMASK = 24;
        _this.IPV6_DEFAULT_NETMASK = 64;
        return _this;
    }
    NetworkSectionService.prototype.init = function () {
        this.networkRepository = network_repository_1.NetworkRepository.getInstance();
        this.networkInterfaceRepository = NetworkInterfaceRepository_1.NetworkInterfaceRepository.getInstance();
        this.systemRepository = system_repository_1.SystemRepository.getInstance();
    };
    NetworkSectionService.prototype.loadEntries = function () {
        var _this = this;
        return this.networkInterfaceRepository.list().then(function (entries) {
            _this.eventDispatcherService.addEventListener(model_event_name_1.ModelEventName.NetworkInterface.listChange, _this.handleEntriesChange.bind(_this));
            return entries;
        });
    };
    NetworkSectionService.prototype.loadExtraEntries = function () {
        var self = this;
        return this.isIpmiLoaded().then(function (isIpmiLoaded) {
            if (isIpmiLoaded) {
                return Promise.all([
                    self.networkRepository.listIpmiChannels()
                ]);
            }
        });
    };
    NetworkSectionService.prototype.loadSettings = function () {
        return Promise.all([
            this.networkRepository.getNetworkSettings(),
            this.systemRepository.getGeneral(),
            this.networkRepository.listNetworkHosts(),
            this.networkRepository.listNetworkStaticRoutes()
        ]).spread(function (settings, general, hosts, routes) {
            settings.system = general;
            settings.hosts = hosts;
            settings.routes = routes;
            return settings;
        });
    };
    NetworkSectionService.prototype.loadOverview = function () {
        return Promise.all([
            this.networkRepository.getNetworkOverview(),
            this.systemRepository.getGeneral()
        ]).spread(function (overview, general) {
            overview.system = general;
            return overview;
        });
    };
    NetworkSectionService.prototype.getNextSequenceForStream = function (streamId) { };
    NetworkSectionService.prototype.saveSettings = function (settings) {
        var _this = this;
        return this.networkRepository.saveNetworkSettings(settings.config).then(function (task) { return task.taskPromise; }).then(function () { return _this.systemRepository.saveGeneral(settings.system); });
    };
    NetworkSectionService.prototype.getNewInterfaceWithType = function (interfaceType) {
        return this.networkInterfaceRepository.getNewInterfaceWithType(interfaceType);
    };
    NetworkSectionService.prototype.initializeInterface = function (networkInterface) {
        var alias;
        networkInterface._networkInterfaces = this.entries;
        networkInterface._otherAliases = [];
        networkInterface._ipAddress = null;
        networkInterface._ipv6Address = null;
        if (networkInterface.dhcp) {
            networkInterface._dhcpAliases = networkInterface.status.aliases;
            for (var i = 0, length_1 = networkInterface.status.aliases.length; i < length_1; i++) {
                alias = networkInterface.status.aliases[i];
                if (alias.type === NetworkInterfaceAliasType_1.NetworkInterfaceAliasType.INET) {
                    networkInterface._dhcpAddress = networkInterface._ipAddress = alias;
                    break;
                }
            }
            for (var j = 0, len = networkInterface.status.aliases.length; j < len; j++) {
                alias = networkInterface.status.aliases[j];
                if (alias.type === NetworkInterfaceAliasType_1.NetworkInterfaceAliasType.INET6) {
                    networkInterface._ipv6Address = alias;
                    break;
                }
            }
        }
        if (networkInterface._ipv6Address === null) {
            networkInterface._ipv6Address = {};
        }
        return this.networkRepository.isClientInterface(networkInterface).then(function (isClient) { return networkInterface._isClient = isClient; });
    };
    NetworkSectionService.prototype.getNewNetworkInterface = function () {
        return this.networkInterfaceRepository.getNewInstance();
    };
    NetworkSectionService.prototype.handleDhcpChangeOnInterface = function (networkInterface) {
        if (networkInterface.dhcp) {
            networkInterface._ipAddress = networkInterface._dhcpAddress;
        }
        else if (!networkInterface.aliases) {
            networkInterface._ipAddress = null;
            networkInterface._ipv6Address = null;
            networkInterface.aliases = [];
        }
        return !networkInterface.dhcp;
    };
    NetworkSectionService.prototype.saveInterface = function (networkInterface) {
        if (networkInterface.type === NetworkInterfaceType_1.NetworkInterfaceType.VLAN) {
            this.cleanupVlanInterface(networkInterface);
        }
        return this.networkInterfaceRepository.save(networkInterface);
    };
    NetworkSectionService.prototype.loadStaticRoutes = function () {
        return this.networkRepository.listNetworkStaticRoutes();
    };
    NetworkSectionService.prototype.getNewStaticRoute = function () {
        return this.networkRepository.getNewNetworkStaticRoute();
    };
    NetworkSectionService.prototype.saveStaticRoute = function (route) {
        return this.networkRepository.saveNetworkStaticRoute(route);
    };
    NetworkSectionService.prototype.deleteStaticRoute = function (route) {
        return this.networkRepository.deleteNetworkStaticRoute(route);
    };
    NetworkSectionService.prototype.loadHosts = function () {
        return this.networkRepository.listNetworkHosts();
    };
    NetworkSectionService.prototype.getNewHost = function () {
        return this.networkRepository.getNewNetworkHost();
    };
    NetworkSectionService.prototype.saveHost = function (host) {
        return this.networkRepository.saveNetworkHost(host);
    };
    NetworkSectionService.prototype.deleteHost = function (host) {
        return this.networkRepository.deleteNetworkHost(host);
    };
    NetworkSectionService.prototype.renewLease = function () {
        var _this = this;
        return this.networkInterfaceRepository.list().then(function (interfaces) {
            return Promise.all(_.compact(_.map(interfaces, function (nic) { return nic.dhcp ? _this.networkInterfaceRepository.renew(nic) : null; })));
        });
    };
    NetworkSectionService.prototype.cleanupVlanInterface = function (networkInterface) {
        if (typeof networkInterface.vlan.tag !== 'number') {
            networkInterface.vlan = {
                tag: null,
                parent: null
            };
        }
    };
    NetworkSectionService.prototype.isIpmiLoaded = function () {
        return this.networkRepository.isIpmiLoaded();
    };
    NetworkSectionService.prototype.handleEntriesChange = function (state) {
        this.dataObjectChangeService.handleDataChange(this.entries, state);
    };
    return NetworkSectionService;
}(abstract_section_service_ng_1.AbstractSectionService));
exports.NetworkSectionService = NetworkSectionService;
